#!/usr/bin/perl

use strict;

use XML::RSS;
use LWP::UserAgent;
use Encode;
use Data::Dumper; 


# ==== settings ====
my $template = "/fido/rss2mail/template.ini";
my $dupefile = "/fido/rss2mail/rss.dupebase";
my $password = "";
my $robotname = "RSSrobot";
my $pktfrom = "2:5019/40";
my $pktto = "2:5019/40";
my $addrfrom = "2:5019/40";
my $tearline = "RSSRobot on GaNJaNET";
my $origin = "GaNJaNET STaTi0N";
my $mail_program = "/fido/hpt/txt2pkt -p $password -xf $pktfrom -xt $pktto -af $addrfrom -nf '$robotname' -nt All -e 'TARGET' -t '$tearline' -o '$origin' -s 'SUBJECT' -d /fido/inbound -";
my $title_charset = "cp866";
my $body_charset = "cp866";
# === that's all ===

my $maintitle; my $description; my $title; my $date; my $url; my $body; my $realbody;
my @dupebase; my $duperec; my $bayan;

die "Usage: rss2mail.pl <URL> <target>" unless ( scalar $ARGV[0] && scalar $ARGV[1]);

open DUPEBASE, "< $dupefile" or die "Can't open dupebase file $dupefile";
while ($duperec = <DUPEBASE>)
{
 chomp $duperec;
 push @dupebase, $duperec;
}
close DUPEBASE;

my $feed = getfeed($ARGV[0]);
die "Can't get RSS feed" unless scalar $feed;

my $rss = new XML::RSS;
$rss->parse($feed);
$maintitle = transcode($rss->{channel}->{title},$title_charset);
$maintitle =~ s/^ +\[[0-9]+\/[0-9]+\] +//g;
$description = transcode($rss->{channel}->{description},$title_charset);
$description =~ s/^ +\[[0-9]+\/[0-9]+\] +//g;
@{$rss->{'items'}} = reverse @{$rss->{'items'}};
for my $item (@{$rss->{items}})
{
 $bayan = 0;
 $title = transcode($item->{'title'},$title_charset);
 $title =~ s/^ +\[[0-9]+\/[0-9]+\] +//g;
 $date = $item->{'pubDate'};
 $url = $item->{'link'};
 $body = $item->{'description'};
 $body = transcode($body,$body_charset);
 $realbody='';
 open (TEMPLATE, $template) || die "Template not found!";
 while (<TEMPLATE>) {
	s/\%title\%/$maintitle/s;
	s/\%description\%/$description/s;
	s/\%date\%/$date/s;
	s/\%url\%/$url/s;
	s/\%text\%/$body/s;
	$realbody.=$_;
 }
 close (TEMPLATE);
 $body=$realbody;
 for $duperec (@dupebase)
 {
  if ($duperec eq $url)
  {
   $bayan = 1;
   last;
  }
 }
 next if $bayan;
 push @dupebase, $url;
 open DUPEBASE, "+>> $dupefile" or die "Can't open dupebase file $dupefile";
 print DUPEBASE "$url\n";
 close DUPEBASE;

 my $mail_process = $mail_program;
 $mail_process =~ s/TARGET/$ARGV[1]/g;
 $mail_process =~ s/SUBJECT/$title/g;

 open OUTPUT, "| $mail_process";
# print OUTPUT "$url\n\n$body\n";
 print OUTPUT "$body";
 close OUTPUT;
}

sub getfeed
{
 my $ua; my $len; my $res;
 $ua = LWP::UserAgent->new;
 $ua->timeout(10);
 $ua->agent("Mozilla/5.0 (some sort of)");
 $res = $ua->get(shift);
 return $res->is_success?$res->content:undef;
}

sub transcode
{
 my $text=shift;
 my $codepage=shift;
 $text =~ s/\<br\>/\n/ig;
 $text =~ s/<(?:[^>'"]*|(['"]).*?\1)*>//sg ;
 $text =~ s/\&quot;/\"/sg;
 $text =~ s/\&nbsp;/\ /sg;
 $text =~ s/\&mdash;/\-/sg;
 $text =~ s/\&ndash;/\-/sg;
 $text =~ s/\&lt;/\</sg;
 $text =~ s/\&gt;/\>/sg;
 $text =~ s/\&amp;/\&/sg;
 $text =~ s/\&laquo;/\"/sg;
 $text =~ s/\&raquo;/\"/sg;
 $text =~ s/\&ldquo;/\"/sg;
 $text =~ s/\&rdquo;/\"/sg;
 $text =~ s/\&lsquo;/\x60/sg;
 $text =~ s/\&rsquo;/\'/sg;
 $text =~ s/\&hellip;/.../sg;
 $text =~ s/x97/-/sg;
 $text =~ s//\"/sg;
 $text =~ s//\"/sg;
 $text =~ s/\&#8206;/\ /sg;
 $text = HTML::Entities::decode_entities($text);
 return Encode::encode($codepage,$text);
}
